/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.notification.Transformer;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.addressing.definition.WSAddressingFactory;
import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.ReferenceParametersType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.WSNotificationFactory;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.WsnbConstants;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.NotificationMessageHolderType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Notify;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Unsubscribe;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.WsnbrConstants;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisher;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisherResponse;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationReader;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.api.ResourcesUuidType;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * Transform a notification to assure the marshalling/unmarshalling between the
 * bus and the external world
 * 
 * @author Frederic Gardes
 * 
 */
public class ComponentTransformer implements Transformer {

    /**
     * The notification broker address for the broker provider
     */
    private String notificationBrokerReferenceAddress = null;

    /**
     * The notification consumer SOA parameter
     */
    private SOAParameterType notificationConsumerSOAParameter = null;

    /**
     * The notification provider SOA parameter
     */
    private SOAParameterType notificationProducerSOAParameter = null;

    /**
     * The publisher registration manager address for the broker provider
     */
    private String publisherRegistrationManagerReferenceAddress = null;

    /**
     * The subscription manager address for the broker provider
     */
    private String subscriptionManagerReferenceAddress = null;

    /**
     * The subscription manager SOA parameter
     */
    private SOAParameterType subscriptionManagerSOAParameter = null;

    public ComponentTransformer(final SOAParameterType notificationProducerSOAParameter,
            final SOAParameterType subscriptionManagerSOAParameter,
            final SOAParameterType notificationConsumerSOAParameter) throws WSNotificationException {
        super();
        this.notificationBrokerReferenceAddress = Constants.Notification.CDK_ADDRESS;
        this.publisherRegistrationManagerReferenceAddress = Constants.Notification.CDK_ADDRESS;
        this.subscriptionManagerReferenceAddress = Constants.Notification.CDK_ADDRESS;
        this.notificationProducerSOAParameter = notificationProducerSOAParameter;
        this.notificationConsumerSOAParameter = notificationConsumerSOAParameter;
        this.subscriptionManagerSOAParameter = subscriptionManagerSOAParameter;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * addNotificationConsumerSOAParameter
     * (com.ebmwebsourcing.wsstar.notification
     * .definition.basenotification.api.Subscribe)
     */
    public Subscribe addNotificationConsumerSOAParameter(final Subscribe subscribe)
    throws WSNotificationException, WSNotificationExtensionException,
    WSAddressingException, MessagingException {
        EndpointReferenceType endpointReferenceType = null;
        ReferenceParametersType referenceParametersType = null;
        if (this.notificationConsumerSOAParameter != null) {
            endpointReferenceType = subscribe.getConsumerReference();
            if (endpointReferenceType == null) {
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                endpointReferenceType.setAddress(Constants.Notification.CDK_ADDRESS);
            }
            referenceParametersType = endpointReferenceType.getReferenceParameters();
            if (referenceParametersType == null) {
                referenceParametersType = endpointReferenceType.newReferenceParameters();
            }
            // Endpoint
            WsnSpecificTypeHelper.setSOAParameter(this.notificationConsumerSOAParameter,
                    referenceParametersType);
            endpointReferenceType.setReferenceParameters(referenceParametersType);
            subscribe.setConsumerReference(endpointReferenceType);
        }
        return subscribe;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * addNotificationProducerSOAParameter
     * (com.ebmwebsourcing.wsstar.notification
     * .definition.basenotification.api.NotificationMessageHolderType)
     */
    public NotificationMessageHolderType addNotificationProducerSOAParameter(
            final NotificationMessageHolderType notificationMessageHolderType)
    throws WSNotificationException, WSAddressingException, MessagingException,
    WSNotificationExtensionException {
        EndpointReferenceType endpointReferenceType = null;
        ReferenceParametersType referenceParametersType = null;
        if (this.notificationProducerSOAParameter != null) {
            endpointReferenceType = notificationMessageHolderType.getProducerReference();
            if (endpointReferenceType == null) {
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                endpointReferenceType.setAddress(Constants.Notification.CDK_ADDRESS);
            }
            referenceParametersType = endpointReferenceType.getReferenceParameters();
            if (referenceParametersType == null) {
                referenceParametersType = endpointReferenceType.newReferenceParameters();
            }
            // Endpoint
            WsnSpecificTypeHelper.setSOAParameter(this.notificationProducerSOAParameter,
                    referenceParametersType);
            endpointReferenceType.setReferenceParameters(referenceParametersType);
            notificationMessageHolderType.setProducerReference(endpointReferenceType);
        }
        return notificationMessageHolderType;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * addNotificationProducerSOAParameter
     * (com.ebmwebsourcing.wsstar.notification
     * .definition.brokerednotification.api.RegisterPublisher)
     */
    public RegisterPublisher addNotificationProducerSOAParameter(
            final RegisterPublisher registerPublisher) throws WSNotificationException,
            WSAddressingException, MessagingException, WSNotificationExtensionException {
        EndpointReferenceType endpointReferenceType = null;
        ReferenceParametersType referenceParametersType = null;
        if (this.notificationProducerSOAParameter != null) {
            endpointReferenceType = registerPublisher.getPublisherReference();
            if (endpointReferenceType == null) {
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                endpointReferenceType.setAddress(Constants.Notification.CDK_ADDRESS);
            }
            referenceParametersType = endpointReferenceType.getReferenceParameters();
            if (referenceParametersType == null) {
                referenceParametersType = endpointReferenceType.newReferenceParameters();
            }
            // Endpoint
            WsnSpecificTypeHelper.setSOAParameter(this.notificationProducerSOAParameter,
                    referenceParametersType);
            endpointReferenceType.setReferenceParameters(referenceParametersType);
            registerPublisher.setPublisherReference(endpointReferenceType);
        }
        return registerPublisher;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * addSubscriptionManagerSOAParameter
     * (com.ebmwebsourcing.wsstar.notification.
     * definition.basenotification.api.NotificationMessageHolderType)
     */
    public NotificationMessageHolderType addSubscriptionManagerSOAParameter(
            final NotificationMessageHolderType notificationMessageHolderType)
    throws WSNotificationException, WSAddressingException, MessagingException,
    WSNotificationExtensionException {
        EndpointReferenceType endpointReferenceType = null;
        ReferenceParametersType referenceParametersType = null;
        if (this.subscriptionManagerSOAParameter != null) {
            endpointReferenceType = notificationMessageHolderType.getSubscriptionReference();
            if (endpointReferenceType == null) {
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                endpointReferenceType.setAddress(Constants.Notification.CDK_ADDRESS);
            }
            referenceParametersType = endpointReferenceType.getReferenceParameters();
            if (referenceParametersType == null) {
                referenceParametersType = endpointReferenceType.newReferenceParameters();
            }
            // Endpoint
            WsnSpecificTypeHelper.setSOAParameter(this.subscriptionManagerSOAParameter,
                    referenceParametersType);
            endpointReferenceType.setReferenceParameters(referenceParametersType);
            notificationMessageHolderType.setSubscriptionReference(endpointReferenceType);
        }
        return notificationMessageHolderType;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * addSubscriptionManagerSOAParameter
     * (com.ebmwebsourcing.wsstar.notification.
     * definition.basenotification.api.SubscribeResponse)
     */
    public SubscribeResponse addSubscriptionManagerSOAParameter(
            final SubscribeResponse subscribeResponse) throws WSNotificationException,
            WSAddressingException, MessagingException, WSNotificationExtensionException {
        EndpointReferenceType endpointReferenceType = null;
        ReferenceParametersType referenceParametersType = null;
        if (this.subscriptionManagerSOAParameter != null) {
            endpointReferenceType = subscribeResponse.getSubscriptionReference();
            if (endpointReferenceType == null) {
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                endpointReferenceType.setAddress(Constants.Notification.CDK_ADDRESS);
            }
            referenceParametersType = endpointReferenceType.getReferenceParameters();
            if (referenceParametersType == null) {
                referenceParametersType = endpointReferenceType.newReferenceParameters();
            }
            // Endpoint
            WsnSpecificTypeHelper.setSOAParameter(this.subscriptionManagerSOAParameter,
                    referenceParametersType);
            endpointReferenceType.setReferenceParameters(referenceParametersType);
            subscribeResponse.setSubscriptionReference(endpointReferenceType);
        }
        return subscribeResponse;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.Transformer#
     * setNotificationBrokerReferenceAddress(java.lang.String)
     */
    public void setNotificationBrokerReferenceAddress(
            final String notificationBrokerReferenceAddress) {
        this.notificationBrokerReferenceAddress = notificationBrokerReferenceAddress;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.Transformer#
     * setPublisherRegistrationManagerReferenceAddress(java.lang.String)
     */
    public void setPublisherRegistrationManagerReferenceAddress(
            final String publisherRegistrationManagerReferenceAddress) {
        this.publisherRegistrationManagerReferenceAddress = publisherRegistrationManagerReferenceAddress;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.Transformer#
     * setSubscriptionManagerReferenceAddress(java.lang.String)
     */
    public void setSubscriptionManagerReferenceAddress(
            final String subscriptionManagerReferenceAddress) {
        this.subscriptionManagerReferenceAddress = subscriptionManagerReferenceAddress;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.Transformer#
     * transformRequestOnConsume(javax.jbi.messaging.NormalizedMessage,
     * javax.xml.namespace.QName)
     */
    public NormalizedMessage transformRequestOnConsume(final NormalizedMessage normalizedMessage,
            final QName operation) throws PEtALSCDKException, WSNotificationException,
            WSAddressingException, MessagingException, WSNotificationExtensionException {
        Document document = null;

        if (WsnbrConstants.REGISTER_PUBLISHER_QNAME.equals(operation)) {
            document = UtilFactory.getSourceUtil().createDocument(normalizedMessage.getContent());
            final RegisterPublisher registerPublisher = this
            .addNotificationProducerSOAParameter(WSNotificationReader.getInstance()
                    .readRegisterPublisher(document));
            document = WSNotificationWriter.getInstance().writeRegisterPublisher(registerPublisher);
            normalizedMessage.setContent(UtilFactory.getSourceUtil().createDOMSource(document));
        } else if (WsnbrConstants.DESTROY_REGISTRATION_QNAME.equals(operation)) {
            // nothing to do now
        } else if (WsnbConstants.SUBSCRIBE_QNAME.equals(operation)) {
            document = UtilFactory.getSourceUtil().createDocument(normalizedMessage.getContent());
            final Subscribe subscribe = this
            .addNotificationConsumerSOAParameter(WSNotificationReader.getInstance()
                    .readSubscribe(document));
            document = WSNotificationWriter.getInstance().writeSubscribe(subscribe);
            normalizedMessage.setContent(UtilFactory.getSourceUtil().createDOMSource(document));
        } else if (WsnbConstants.NOTIFY_QNAME.equals(operation)) {
            document = UtilFactory.getSourceUtil().createDocument(normalizedMessage.getContent());
            final Notify notify = WSNotificationReader.getInstance().readNotify(document);
            final List<NotificationMessageHolderType> notificationMessageList = new ArrayList<NotificationMessageHolderType>(
                    notify.getNotificationMessage().size());
            for (NotificationMessageHolderType notificationMessageHolderType : notify
                    .getNotificationMessage()) {
                notificationMessageHolderType = this
                .addSubscriptionManagerSOAParameter(notificationMessageHolderType);
                notificationMessageHolderType = this
                .addNotificationProducerSOAParameter(notificationMessageHolderType);
                notificationMessageList.add(notificationMessageHolderType);
            }
            notify.getNotificationMessage().clear();
            notify.getNotificationMessage().addAll(notificationMessageList);
            document = WSNotificationWriter.getInstance().writeNotify(notify);
            normalizedMessage.setContent(UtilFactory.getSourceUtil().createDOMSource(document));
        }
        return normalizedMessage;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * transformRequestOnProducer
     * (org.ow2.petals.component.framework.api.notification.notify.Notify)
     */
    public Notify transformRequestOnProducer(final Notify notify) throws WSNotificationException,
    WSAddressingException {
        EndpointReferenceType endpointReferenceType = null;

        final List<NotificationMessageHolderType> notificationMessageList = new ArrayList<NotificationMessageHolderType>(
                notify.getNotificationMessage().size());
        for (final NotificationMessageHolderType notificationMessageHolderType : notify
                .getNotificationMessage()) {
            endpointReferenceType = notificationMessageHolderType.getSubscriptionReference();
            if (endpointReferenceType != null) {
                // Subscription Reference
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                // Address
                endpointReferenceType.setAddress(this.subscriptionManagerReferenceAddress);
                notificationMessageHolderType.setSubscriptionReference(endpointReferenceType);
            }
            endpointReferenceType = notificationMessageHolderType.getProducerReference();
            // producer reference
            if (endpointReferenceType != null) {
                endpointReferenceType.setAddress(notificationMessageHolderType
                        .getProducerReference().getAddress());
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                // Address
                endpointReferenceType.setAddress(this.notificationBrokerReferenceAddress);
                notificationMessageHolderType.setProducerReference(endpointReferenceType);
            }
            // adding to the new list
            notificationMessageList.add(notificationMessageHolderType);
        }
        notify.getNotificationMessage().clear();
        notify.getNotificationMessage().addAll(notificationMessageList);
        return notify;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * transformRequestOnProducer
     * (com.ebmwebsourcing.wsstar.notification.definition
     * .basenotification.api.Subscribe)
     */
    public Subscribe transformRequestOnProducer(final Subscribe subscribe)
    throws WSAddressingException {
        EndpointReferenceType endpointReferenceType = null;

        // Consumer Reference mandatory: no control
        endpointReferenceType = WSAddressingFactory.getInstance().newEndpointReferenceType();
        // Address
        endpointReferenceType.setAddress(this.notificationBrokerReferenceAddress);
        subscribe.setConsumerReference(endpointReferenceType);
        return subscribe;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * transformRequestOnProducer
     * (com.ebmwebsourcing.wsstar.notification.definition
     * .basenotification.api.Unsubscribe)
     */
    public Unsubscribe transformRequestOnProducer(Unsubscribe unsubscribe)
    throws WSNotificationException, WSNotificationExtensionException {
        ResourcesUuidType resourcesUuid = null;

        // keep resource id
        resourcesUuid = WsnSpecificTypeHelper.getResourcesUuidType(unsubscribe);
        unsubscribe = WSNotificationFactory.getInstance().createUnsubscribe();
        WsnSpecificTypeHelper.setResourcesUuidType(resourcesUuid, unsubscribe);
        return unsubscribe;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.Transformer#
     * transformResponseOnConsume(javax.jbi.messaging.NormalizedMessage,
     * javax.xml.namespace.QName)
     */
    public NormalizedMessage transformResponseOnConsume(final NormalizedMessage normalizedMessage,
            final QName operation) throws PEtALSCDKException, WSNotificationException,
            WSAddressingException, MessagingException, WSNotificationExtensionException {
        Document document = null;
        EndpointReferenceType endpointReferenceType = null;
        ReferenceParametersType referenceParametersType = null;
        ResourcesUuidType resourcesUuid = null;

        if (WsnbrConstants.REGISTER_PUBLISHER_QNAME.equals(operation)) {
            document = UtilFactory.getSourceUtil().createDocument(normalizedMessage.getContent());
            final RegisterPublisherResponse registerPublisherResponse = WSNotificationReader
            .getInstance().readRegisterPublisherResponse(document);
            // Publisher Registration Reference
            endpointReferenceType = registerPublisherResponse.getPublisherRegistrationReference();
            if (endpointReferenceType != null) {
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                // Address
                endpointReferenceType.setAddress(this.publisherRegistrationManagerReferenceAddress);
                // Resource
                resourcesUuid = WsnSpecificTypeHelper
                .getResourcesUuidType(registerPublisherResponse
                        .getPublisherRegistrationReference().getReferenceParameters());
                referenceParametersType = endpointReferenceType.getReferenceParameters();
                WsnSpecificTypeHelper.setResourcesUuidType(resourcesUuid, referenceParametersType);
                endpointReferenceType.setReferenceParameters(referenceParametersType);
                registerPublisherResponse.setPublisherRegistrationReference(endpointReferenceType);
            }
            // Consumer Reference
            endpointReferenceType = registerPublisherResponse.getConsumerReference();
            if (endpointReferenceType != null) {
                endpointReferenceType = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                // Address
                endpointReferenceType.setAddress(this.notificationBrokerReferenceAddress);
                registerPublisherResponse.setConsumerReference(endpointReferenceType);
            }
            document = WSNotificationWriter.getInstance().writeRegisterPublisherResponse(
                    registerPublisherResponse);
            normalizedMessage.setContent(UtilFactory.getSourceUtil().createDOMSource(document));
        } else if (WsnbrConstants.DESTROY_REGISTRATION_QNAME.equals(operation)) {
            // nothing to do now
        } else if (WsnbConstants.SUBSCRIBE_QNAME.equals(operation)) {
            document = UtilFactory.getSourceUtil().createDocument(normalizedMessage.getContent());
            final SubscribeResponse subscribeResponse = WSNotificationReader.getInstance()
            .readSubscribeResponse(document);
            // Subscription Reference mandatory
            endpointReferenceType = WSAddressingFactory.getInstance().newEndpointReferenceType();
            // Address
            endpointReferenceType.setAddress(this.subscriptionManagerReferenceAddress);
            // Resource
            resourcesUuid = WsnSpecificTypeHelper.getResourcesUuidType(subscribeResponse
                    .getSubscriptionReference().getReferenceParameters());
            referenceParametersType = endpointReferenceType.getReferenceParameters();
            WsnSpecificTypeHelper.setResourcesUuidType(resourcesUuid, referenceParametersType);
            endpointReferenceType.setReferenceParameters(referenceParametersType);
            subscribeResponse.setSubscriptionReference(endpointReferenceType);
            document = WSNotificationWriter.getInstance().writeSubscribeResponse(subscribeResponse);
            normalizedMessage.setContent(UtilFactory.getSourceUtil().createDOMSource(document));
        }
        return normalizedMessage;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.Transformer#
     * transformResponseOnProducer
     * (com.ebmwebsourcing.wsstar.notification.definition
     * .basenotification.api.SubscribeResponse)
     */
    public SubscribeResponse transformResponseOnProducer(final SubscribeResponse subscribeResponse)
    throws PEtALSCDKException, WSNotificationException, WSAddressingException,
    MessagingException, WSNotificationExtensionException {
        return this.addSubscriptionManagerSOAParameter(subscribeResponse);
    }
}
