/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification;

import java.util.Iterator;
import java.util.List;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;

/**
 * @author Frederic Gardes
 *
 */
public class NotificationServiceEndpoint implements ServiceEndpoint {

    private String endpoint = null;

    private QName service = null;

    private List<QName> interfaceList = null;

    /**
     * The constructor
     * 
     * @param interfaceList
     *            The list of interfaces
     * @param service
     *            The service
     * @param endpoint
     *            The endpoint
     */
    public NotificationServiceEndpoint(final List<QName> interfaceList, final QName service, final String endpoint) {
        this.interfaceList = interfaceList;
        this.service = service;
        this.endpoint = endpoint;
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getAsReference(javax.xml.namespace.QName)
     */
    public DocumentFragment getAsReference(final QName operationName) {
        Element element = null;
        DocumentFragment result = null;

        try {
            // TODO not create a lot of time the factory and the builder
            final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder;
            documentBuilder = documentBuilderFactory.newDocumentBuilder();
            final Document document = documentBuilder.newDocument();
            final Element root = document.createElement("notificationServiceEndpoint");
            if (this.interfaceList != null) {
                final Iterator<QName> iterator = this.interfaceList.iterator();
                if (iterator != null) {
                    while (iterator.hasNext()) {
                        final QName notificationServiceEndpointInterface = iterator.next();
                        if (notificationServiceEndpointInterface != null) {
                            if (notificationServiceEndpointInterface.getNamespaceURI() != null) {
                                element = document.createElementNS(notificationServiceEndpointInterface
                                        .getNamespaceURI(), "interface");
                            }
                            if (notificationServiceEndpointInterface.getLocalPart() != null) {
                                element.setTextContent(notificationServiceEndpointInterface
                                        .getLocalPart());
                            }
                            root.appendChild(element);
                        }
                    }
                }
            }
            if (this.service != null) {
                if (this.service.getNamespaceURI() != null) {
                    element = document.createElementNS(this.service.getNamespaceURI(), "service");
                }
                if (this.service.getLocalPart() != null) {
                    element.setTextContent(this.service.getLocalPart());
                }
                root.appendChild(element);
            }
            if (this.endpoint != null) {
                element = document.createElement("endpoint");
                element.setTextContent(this.endpoint);
                root.appendChild(element);
            }
            result = document.createDocumentFragment();
        } catch (final ParserConfigurationException e) {
            // nothing to do
        }
        return result;
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getEndpointName()
     */
    public String getEndpointName() {
        return this.endpoint;
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getInterfaces()
     */
    public QName[] getInterfaces() {
        return this.interfaceList.toArray(new QName[this.interfaceList.size()]);
    }

    /* (non-Javadoc)
     * @see javax.jbi.servicedesc.ServiceEndpoint#getServiceName()
     */
    public QName getServiceName() {
        return this.service;
    }

}
