/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.subscriptionpolicy;

import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ow2.petals.component.framework.api.notification.subscriptionpolicy.SubscriptionPolicy;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscriptionPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.api.XsltStyleSheetType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * A subscription policy to transform the initial document to another applying a
 * XSL stylesheet
 * 
 * @author Frederic Gardes
 * 
 */
public class XSLTSubscriptionPolicy implements SubscriptionPolicy {

    private Logger logger = null;

    private Transformer transformer = null;

    public XSLTSubscriptionPolicy(final Subscribe subscribe, final Logger logger)
    throws WSNotificationExtensionException, TransformerFactoryConfigurationError,
    TransformerException, ParserConfigurationException {
        super();
        this.logger = logger;
        final SubscriptionPolicyType subscriptionPolicy = subscribe.getSubscriptionPolicy();
        if (subscriptionPolicy != null) {
            final XsltStyleSheetType xsltStyleSheet = WsnSpecificTypeHelper
            .getXsltStyleSheetType(subscriptionPolicy);
            if (xsltStyleSheet != null) {
                final Element element = xsltStyleSheet.getStyleSheet();
                if (element != null) {
                    final Source source = new DOMSource(element);
                    if (source != null) {
                        this.transformer = UtilFactory.getXSLTUtil().createTransformer(source);
                    }
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.subscriptionpolicy.
     * SubscriptionPolicy
     * #applyPolicy(com.ebmwebsourcing.wsstar.notification.definition
     * .basenotification.api.SubscriptionPolicyType, org.w3c.dom.Document)
     */
    public Document applyPolicy(Document document) {
        try {
            if (this.transformer != null) {
                if (this.logger.isLoggable(Level.FINE)) {
                    this.logger.fine("original document: " + this.toString(document));
                }
                document = UtilFactory.getXSLTUtil().transformDocument(document, this.transformer);
                if (this.logger.isLoggable(Level.FINE)) {
                    this.logger.fine("transformed document: " + this.toString(document));
                }
            }
        } catch (final TransformerException e) {
            // nothing to do
        } catch (final ParserConfigurationException e) {
            // nothing to do
        }
        return document;
    }

    /**
     * Convert to string a document
     * 
     * @param document
     *            The document
     * @return The document as String
     * @throws TransformerException
     */
    private String toString(final Document document) throws TransformerException {
        final DOMSource domSource = new DOMSource(document);
        final StringWriter writer = new StringWriter();
        final StreamResult result = new StreamResult(writer);
        final TransformerFactory tf = TransformerFactory.newInstance();
        final Transformer transformer = tf.newTransformer();
        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.transform(domSource, result);
        transformer.reset();
        return writer.toString();
    }
}
