/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ------------------------------------------------------------------------- $Id
 * $ -------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.util;

import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;

import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.api.util.ExchangeUtil;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * Contains utilities methods for Exchange operations.
 * 
 * @author nsalatge - EBMWebsourcing
 * @since 4.0
 * 
 */
public final class ExchangeUtilImpl implements ExchangeUtil {
    /**
     * Constructor
     */
    public ExchangeUtilImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.ExchangeUtil#copyExchange
     * (javax.jbi.messaging.MessageExchangeFactory,
     * org.ow2.petals.component.framework.api.message.Exchange)
     */
    public Exchange copyExchange(final MessageExchangeFactory mef, final Exchange source)
            throws MessagingException, PEtALSCDKException {

        final MessageExchange msgEx = mef.createExchange(source.getPattern());
        final Exchange target = new ExchangeImpl(msgEx);

        // Set all values in concrete message exchange to fake message exchange
        if (source.getEndpoint() != null) {
            target.setEndpoint(source.getEndpoint());
        }
        target.setInterfaceName(source.getInterfaceName());
        target.setService(source.getService());
        target.setOperation(source.getOperation());
        target.setStatus(source.getStatus());

        if (source.getError() != null) {
            target.setError(source.getError());
        }

        if (source.getFault() != null) {
            target.setFault(source.getFault());
        }

        if (source.getInMessage() != null && source.getInMessage().getContent() != null) {
            this.copyNormalizedMessage(source.getInMessage(), target.getInMessage());
        }

        if (source.getOutMessage() != null && source.getOutMessage().getContent() != null) {
            this.copyNormalizedMessage(source.getOutMessage(), target.getOutMessage());
        }
        this.copyExchangeProperties(source, target);
        return target;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.util.IExchangeUtil#
     * copyNormalizedMessage(javax.jbi.messaging.NormalizedMessage,
     * javax.jbi.messaging.NormalizedMessage)
     */
    @SuppressWarnings("unchecked")
    public void copyNormalizedMessage(final NormalizedMessage originalNm,
            final NormalizedMessage copyNm) throws MessagingException, PEtALSCDKException {
        // setting content, with hardcopy (because if semantic routing...)
        final Document contentDoc = UtilFactory.getSourceUtil().createDocument(
                originalNm.getContent());
        final Source source = UtilFactory.getSourceUtil().createDOMSource(contentDoc);
        copyNm.setContent(source);
        // a copy is not a consume : set again the original content read
        originalNm.setContent(source);
        // setting properties
        final Set<String> propertyNames = originalNm.getPropertyNames();
        for (final String propertyName : propertyNames) {
            copyNm.setProperty(propertyName, originalNm.getProperty(propertyName));
        }

        // setting attachments if any
        final Set<String> inAttachmentNames = originalNm.getAttachmentNames();
        if (inAttachmentNames != null) {
            for (final String attachmentName : inAttachmentNames) {
                final DataHandler attDh = originalNm.getAttachment(attachmentName);
                copyNm.addAttachment(attachmentName, attDh);
            }
        }

        // setting security context
        copyNm.setSecuritySubject(originalNm.getSecuritySubject());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.ExchangeUtil#isPetalsException
     * (javax.jbi.messaging.Fault)
     */
    public boolean isPetalsException(final Fault fault) throws PEtALSCDKException {
        final NodeList resList = UtilFactory.getSourceUtil().createDocument(fault.getContent())
                .getElementsByTagName("faultactor");
        if (resList.getLength() > 0) {
            return resList.item(0).getTextContent().startsWith(
                    Constants.Exception.NAMESPACE_URI.toString());
        }

        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.util.ExchangeUtil#
     * copyExchangeProperties
     * (org.ow2.petals.component.framework.api.message.Exchange,
     * org.ow2.petals.component.framework.api.message.Exchange)
     */
    public void copyExchangeProperties(final Exchange source, final Exchange target)
            throws MessagingException, PEtALSCDKException {
        if (source.getPropertyNames() != null) {
            for (final Object property : source.getPropertyNames()) {
                if (property instanceof String) {
                    final String key = (String) property;
                    if (key.startsWith(Message.PROPAGATION_PREFIX)) {
                        target.setProperty(key, source.getProperty(key));
                    }
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.ExchangeUtil#isBaseNotification
     * (org.ow2.petals.component.framework.api.message.Exchange)
     */
    public boolean isBaseNotification(final Exchange exchange) {
        if (exchange == null || exchange.getInterfaceName() == null
                || exchange.getInterfaceName().getNamespaceURI() == null) {
            return false;
        }
        return Constants.Notification.WS_BASE_NOTIFICATION_WSDL_NAMESPACE_URI.equals(exchange
                .getInterfaceName().getNamespaceURI());
    }
}
